import java.awt.*;
import java.io.*;
import java.net.*;
import java.awt.event.*;
import java.util.*;


public class SBRecord extends Frame implements Runnable
{
    // GUI objects
    private SBRecordPreview PreviewFrame;
    private SBRecordMain MainFrame;
    private SBDSPGauge DSPGauge;

    // Sockets for instructor vs. driver interaction
    private Socket OutDriverSocket, InDriverSocket;
    private ServerSocket ServerDriverSocket;

    // Streams for instructor vs. driver interaction
    private OutputStream DriverOutputStream;
    private InputStream DriverInputStream;


    // Configuration stuff
    private byte TTL;
    private int mcastPort;
    private String mcastIPString;
    private int driverPort;
    private String driverIP;
    private int serverPort;
    private String PTstring;
    private int PT;

    // Multicast stuff
    private InetAddress mcastIP;
    private MulticastSocket mcastSocket;

    int sequencenum=0;
    int lasttimestamp=1;
    byte input[]=new byte[243]; // Together they make up 255
    byte header[]=new byte[12]; // RTP header
    byte packet[]=new byte[255]; // The actual packet

    public boolean debug=false; // By default, debug mode is off



    // -1 - Exit
    // 0  - Before connecting with the driver
    // 1  - Determine SoftBoard status
    // 2  - Download DSP
    // 3  - Start Recording
    // 4  - Stop Recording
    private int myStatus=0; 


    
    private Thread myThread;

    public static void main(String args[])
    {
	new SBRecord(args);
    }

    public SBRecord(String args[])
    {
	// Check command line args
	if (args.length>0 && args[0].equals("-d"))
	    debug=true;          // Debug mode on

	// Process configuration file
	try {
	    File confFile=new File("SBRecord.conf");
	    if (!confFile.exists())
		{
		    System.out.println("SBRecord.conf is missing");
		    System.exit(1);
		}
	    BufferedReader buffer=new BufferedReader(new FileReader(confFile));

	    // Get multicast TTL string
	    String strTTL=buffer.readLine();
	    strTTL=strTTL.substring(strTTL.indexOf('=')+1);

	    // Get multicast port string
	    String strPort=buffer.readLine();
	    strPort=strPort.substring(strPort.indexOf('=')+1);

	    // Get multicast IP string 
	    String strIP=buffer.readLine();
	    strIP=strIP.substring(strIP.indexOf('=')+1);

	    // Get driver port string
	    String strdPort=buffer.readLine();
	    strdPort=strdPort.substring(strdPort.indexOf('=')+1);

	    // Get driver IP string
	    String strdIP=buffer.readLine();
	    strdIP=strdIP.substring(strdIP.indexOf('=')+1);

	    // Get local server port string
	    String strsPort=buffer.readLine();
	    strsPort=strsPort.substring(strsPort.indexOf('=')+1);

	    // Get PT value for the RTP header (0-127)
	    PTstring=buffer.readLine();
	    PTstring=PTstring.substring(PTstring.indexOf('=')+1);

	    // Convert string configuration info to corresponding types
	    serverPort=(new Integer(strsPort)).intValue();
	    driverPort=(new Integer(strdPort)).intValue();
	    driverIP=strdIP;
	    TTL=(byte)(new Integer(strTTL)).intValue();
	    mcastIPString=strIP;
	    mcastPort=(new Integer(strPort)).intValue();
	    PT=(new Integer(PTstring)).intValue();

	    // Configuration tracer
	    if (debug)
		System.out.println("Configuration processed: \n=================================================\nTTL="+strTTL+";\nMcastport="+strPort+";\nMcastIP="+strIP+";\nDriver Port="+strdPort+";\nDriver IP="+strdIP+";\nServer Port="+strsPort+";\nPT="+PT);
	} catch (Exception e)
	    {
		System.out.println("Error reading conf file");
		System.exit(1);
	    }

	// Create multicast socket
	try {
	    mcastIP = InetAddress.getByName(mcastIPString);
	    mcastSocket = new MulticastSocket();
	} catch (Exception e)
	    {
		System.out.println("Unable to obtain socket: " + e);
		System.exit(1);
	    }

	// Instantiate GUI objects
	MainFrame=new SBRecordMain(this);
	DSPGauge=new SBDSPGauge(MainFrame);
	PreviewFrame = new SBRecordPreview(MainFrame);
	
	// Start the server listener thread
	myThread=new Thread(this);
	myThread.start();

	
	try {

	    // Starting the GUI and connecting to the driver.
	    PreviewFrame.pack();
	    PreviewFrame.setVisible(true);
	    PreviewFrame.inform("Starting the server communication socket for the driver");
	    ServerDriverSocket=new ServerSocket(serverPort);
	    PreviewFrame.inform("Opening a client communication socket with the driver");

	    // Letting the driver know we are ready.
	    OutDriverSocket=new Socket(InetAddress.getByName(driverIP), driverPort);
	    DriverOutputStream=OutDriverSocket.getOutputStream();
	    PreviewFrame.inform("Letting the driver know we are READY");
	    DriverOutputStream.write((new String("READY")).getBytes(), 0, (new String("READY")).length());

      } catch (Exception e)
	{
	    System.out.println(e.toString());
	    PreviewFrame.inform(e.toString());
	    try {
		Thread.currentThread().sleep(5000);
	    } catch (Exception ex)
		{
		}
	    System.exit(1);
	}
	
    }
    
    public void run()
    {
	while (ServerDriverSocket==null)
	    {
		try {
		    Thread.currentThread().sleep(10);
		} catch (Exception e)
		    {
		    }
	    }
	// Wait for the driver to show up
	try {
	    InDriverSocket=ServerDriverSocket.accept();
	    DriverInputStream=InDriverSocket.getInputStream();
	} catch (Exception e)
	    {
		System.out.println("Failed to accept driver socket");
		System.out.println(e.toString());
	    }
	while(myStatus!=-1)
	    {
		// Listen to the driver
		synchronized(this)
		    {
			String inputString;

			try {
			    // Get the message from the driver
			    int len=DriverInputStream.read(input);
			    inputString=new String(input);
			    if (len<inputString.length())
				inputString=inputString.substring(0, len);
			    if (debug)
				System.out.println("--> ["+inputString+"]");

			    if (myStatus==3) 
				{
				    if (debug)
					System.out.println("<<--" + new String(input) + "-->>");
				    
				    // Attach the RTP header to each outgoing packet.
				    attachHeader();
				    
				    // Create the packet
				    DatagramPacket mcastPacket=new DatagramPacket(packet, packet.length, mcastIP, mcastPort);
				    try {
					// Send it.
					mcastSocket.send(mcastPacket, TTL);
				    } catch (Exception e)
					{
					    System.out.println("Failed to send a multicast: " + e.toString());
					}
				}
			    
			    // Process messages from the driver.

			    if (myStatus==0 && inputString.equals("READY"))
				{
				    PreviewFrame.inform("The driver is READY");
				    myStatus=1;
				    PreviewFrame.inform("Requesting SoftBoard Status");
				    DriverOutputStream.write((new String("QUERY_SOFTBOARD_STATUS")).getBytes(), 0, (new String("QUERY_SOFTBOARD_STATUS")).length());	      
				}
			    else if (myStatus==1 && inputString.equals("QUERY_SOFTBOARD_STATUS_NOTACTIVE"))
				{
				    PreviewFrame.inform("SoftBoard is not active");
				    PreviewFrame.inform("Requesting the driver to download DSP code");
				    myStatus=2;
				    DriverOutputStream.write((new String("DOWNLOAD_DSP")).getBytes(), 0, (new String("DOWNLOAD_DSP")).length());
				    try {
					Thread.currentThread().sleep(5000);
				    } catch (Exception e)
					{
					}
				    PreviewFrame.setVisible(false);
				    DSPGauge.pack();
				    DSPGauge.setVisible(true);
				}
			    else if (myStatus==2 && inputString.equals("DOWNLOAD_DSP"))
				{
				    DSPGauge.increment();
				    DSPGauge.repaint();
				    DriverOutputStream.write((new String("DOWNLOAD_DSP_READY")).getBytes(), 0, (new String("DOWNLOAD_DSP_READY")).length());
				}
			    else if ((myStatus==2 && inputString.equals("DOWNLOAD_DSP_DONE")) || (myStatus==1 && inputString.equals("QUERY_SOFTBOARD_STATUS_ACTIVE")))
				{
				    DSPGauge.setVisible(false);
				    PreviewFrame.setVisible(false);
				    MainFrame.setVisible(true);
				    DriverOutputStream.write((new String("SOFTBOARD_EXTENTS")).getBytes(), 0, (new String("SOFTBOARD_EXTENTS")).length());
				    myStatus=4;
				}
			    else if (inputString.startsWith("SOFTBOARD_EXTENTS"))
				{
				    int i, j;
				    String temp;
				    i=inputString.indexOf(':');
				    j=inputString.indexOf(':', i+1);
				    temp=inputString.substring(i+1, j);
				    MainFrame.setLeft((new Integer(temp)).intValue());
				    
				    i=inputString.indexOf(':', j+1);
				    temp=inputString.substring(j+1, i);
				    MainFrame.setRight((new Integer(temp)).intValue());
				    
				    j=inputString.indexOf(':', i+1);
				    temp=inputString.substring(i+1, j);
				    MainFrame.setTop((new Integer(temp)).intValue());
				    
				    i=inputString.indexOf(':', j+1);
				    temp=inputString.substring(j+1, i);
				    MainFrame.setBottom((new Integer(temp)).intValue());
				    
				    temp=inputString.substring(i+1);
				    MainFrame.setWidth((new Integer(temp)).intValue());
				    MainFrame.printExtents();
				    DriverOutputStream.write((new String("STOP_RECORDING")).getBytes(), 0, (new String("STOP_RECORDING")).length());
				}
			    else if (myStatus==3 && inputString.startsWith("PENDOWN"))
				{
				    int i, j;
				    String pen;
				    String time;
				    String x;
				    String y;
				    
				    i=inputString.indexOf(':');
				    j=inputString.indexOf(':', i+1);
				    pen=inputString.substring(i+1, j);
				    
				    i=inputString.indexOf(':', j+1);
				    time=inputString.substring(j+1, i);
				    
				    j=inputString.indexOf(':', i+1);
				    x=inputString.substring(i+1, j);
				    
				    i=inputString.indexOf(':', j+1);
				    y=inputString.substring(j+1, i);
				    MainFrame.penDown(pen, (new Integer(time)).intValue(), (new Integer(x)).intValue(), (new Integer(y)).intValue());
				}
			    else if (myStatus==3 && inputString.startsWith("DELTADRAW"))
				{
				    int i, j;
				    String x;
				    String y;
				    
				    i=inputString.indexOf(':');
				    j=inputString.indexOf(':', i+1);
				    x=inputString.substring(i+1, j);
				    
				    i=inputString.indexOf(':', j+1);
				    y=inputString.substring(j+1, i);
				    
				    MainFrame.deltaDraw((new Integer(x)).intValue(), (new Integer(y)).intValue());
				}
			    else if (myStatus==3 && inputString.startsWith("DRAW"))
				{
				    int i, j;
				    String x;
				    String y;
				    
				    i=inputString.indexOf(':');
				    j=inputString.indexOf(':', i+1);
				    x=inputString.substring(i+1, j);
				    
				    i=inputString.indexOf(':', j+1);
				    y=inputString.substring(j+1, i);
				    
				    MainFrame.Draw((new Integer(x)).intValue(), (new Integer(y)).intValue());
				}
			    else if (myStatus==3 && inputString.startsWith("PENUP"))
				{
				    MainFrame.penUp();
				}
			    Thread.currentThread().sleep(10);
			} catch (Exception e)
			    {
				System.out.println("Non-critical exception: "+e.toString());
			    }
		    }
	    }
    }
    
    public boolean startRecording()
    {
	try {
	    DriverOutputStream.write((new String("START_RECORDING")).getBytes(), 0, (new String("START_RECORDING")).length());
	} catch (Exception e)
	    {
		return false;
	    }
	myStatus=3;
	return true;
    }
    
    public boolean stopRecording()
    {
	try {
	    DriverOutputStream.write((new String("STOP_RECORDING")).getBytes(), 0, (new String("STOP_RECORDING")).length());
	} catch (Exception e)
	    {
		return false;
	    }
	myStatus=4;
	return true;
    }
    
    public void quit()
    {
	try {
	    DriverOutputStream.write((new String("BYE")).getBytes(), 0, (new String("BYE")).length());
	} catch (Exception e)
	    {
	    }
	mcastSocket.close();
	System.exit(1);
    }


    public void attachHeader()
    {
	// The first byte is:
        // [10][0][0][0000], thus altogether 128 or 0x80.
        //  V   P  X   CC 
	header[0]=(new Integer(128)).byteValue();
	
	// The second byte is:
        // [1][xxxxxxx]
	//  M    PT    
	header[1]=(new Integer(128+PT)).byteValue();
	
	// The third and fourth byte is a sequence number
	sequencenum++;
	if (sequencenum>65535) // We can keep only two bytes
	    sequencenum=0; // wrap around

	header[2]=(new Integer(sequencenum/256)).byteValue();
	header[3]=(new Integer(sequencenum%256)).byteValue();
	
	// Let's try to get the timestamp from the input buffer
	String inputString=new String(input);
	
	if (inputString.startsWith("PENDOWN"))
	    {
		int i, j;
		String pen;
		String time;
		
		i=inputString.indexOf(':');
		j=inputString.indexOf(':', i+1);
		pen=inputString.substring(i+1, j);
		
		i=inputString.indexOf(':', j+1);
		time=inputString.substring(j+1, i);

		lasttimestamp=(new Integer(time)).intValue();
	    }
	else lasttimestamp++;
	
	int stamp=lasttimestamp;

	// BIG_ENDIAN
	header[4]=(byte) (stamp >> 24);
	header[5]=(byte) (stamp >> 16);
	header[6]=(byte) (stamp >> 8);
	header[7]=(byte) stamp;


	// Random long integer
	int randomint1=(int)(Math.random()*65536);
	int randomint2=(int)(Math.random()*65536);
	header[8]=(byte) (randomint1 >> 8);
	header[9]=(byte) randomint1;
	header[10]=(byte) (randomint2 >> 8);
	header[11]=(byte) randomint2;
	
	for (int i=0; i<12; i++)
	    packet[i]=header[i];
	for (int i=0; i<243; i++)
	    packet[12+i]=input[i];
    }

}

class SBRecordPreview extends Dialog
{
    private TextArea info=new TextArea(25, 80);
    
    public SBRecordPreview(Frame owner)
    {
	super(owner, "Please Wait", false);
	setSize(500, 500);
	Toolkit t=Toolkit.getDefaultToolkit();
	setLocation((t.getScreenSize().width-500)/2, (t.getScreenSize().height-500)/2);
	info.setEditable(false);
	add(info);
    }
    
    public void inform(String text)
    {
	info.append(text+"\n");
    }
}

class SBRecordMain extends Frame
{
    private Button StartRecording=new Button("Start Recording");
    private Button StopRecording=new Button("Stop Recording");
    private Button Quit=new Button("Quit");
    private SBRecord owner;
    private int left=30, right=4080, top=297, bottom=3334, width=5400;
    private Vector strokes;
    private Stroke current=null;
    
    
    // 0 - no eraser
    // 1 - small eraser
    // 2 - large eraser
    
    public SBRecordMain(SBRecord myowner)
    {
	super("SoftBoard Instructor Application");
	owner=myowner;
	strokes=new Vector(100, 100);
	Toolkit t=Toolkit.getDefaultToolkit();
	setLocation((t.getScreenSize().width-700)/2, (t.getScreenSize().height-520)/2);
	setSize(700, 520);
	setVisible(false);
	setLayout(new BorderLayout());
	
	setBackground(Color.white);
	Panel bottom=new Panel();
	
	StopRecording.setEnabled(false);
	StartRecording.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e)
		{
		    if (owner.startRecording())
			{
			    StartRecording.setEnabled(false);
			    StopRecording.setEnabled(true);
			}
		}
	});
	StopRecording.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e)
		{
		    if (owner.stopRecording())
			{
			    StopRecording.setEnabled(false);
			    StartRecording.setEnabled(true);
			}
		}
	});
	Quit.addActionListener(new ActionListener() {
	    public void actionPerformed(ActionEvent e)
		{
		    owner.quit();
		}
	});
	bottom.add(StartRecording);
	bottom.add(StopRecording);
	bottom.add(Quit);
	add(bottom, "South");
    }
    
    public void setLeft(int left)
    {
	this.left=left;
    }
    
    public void setRight(int right)
    {
	this.right=right;
    }

    public void setTop(int top)
    {
	this.top=top;
    }
    
    public void setBottom(int bottom)
    {
	this.bottom=bottom;
    }
    
    public void setWidth(int width)
    {
	this.width=width;
    }
    
    public void penDown(String color, int t, int x, int y)
    {
	current=new Stroke(t, x, y, color);
	repaint((int)((x-40)*640.0/(right-left)), (int)((y-40)*480.0/(bottom-top)), 80, 80);
    }
    
    public void penUp()
    {
	strokes.addElement((Object)current);
	current=null;
    }
    
    public void deltaDraw(int x, int y)
    {
	StrokeElement e=new StrokeElement(x, y, 0);
	Enumeration elements=current.elements();
	current.add(e);
	int finalx=current.getX(), finaly=current.getY();
	while (elements.hasMoreElements())
	    {
		StrokeElement el=(StrokeElement) elements.nextElement();
		if (el.getType()==0)
		    {
			finalx+=el.getX();
			finaly+=el.getY();
		    }
		else 
		    {
			finalx=el.getX();
			finaly=el.getY();
		    }
	    }
	repaint((int)((finalx-40)*640.0/(right-left)), (int)((finaly-40)*480.0/(bottom-top)), 80, 80);
  }
    
    public void Draw(int x, int y)
    {
	StrokeElement e=new StrokeElement(x, y, 1);
	current.add(e);
	repaint((int)((x-40)*640.0/(right-left)), (int)((y-40)*480.0/(bottom-top)), 80, 80);
    }
    
    public void update(Graphics g)
    {
      paint(g);
    }
    
    public void paint(Graphics g)
    {
	g.clearRect(0, 0, 640, 480);
	Enumeration strokesEnum=strokes.elements();
	while (strokesEnum.hasMoreElements())
	    {
		Color PenColor;
		Stroke s = (Stroke) strokesEnum.nextElement();
		if (s!=null)
		    {
			Enumeration strokeElementsEnum=s.elements();
			if (s.getPen().equals("black"))
			    PenColor=Color.black;
			else if (s.getPen().equals("red"))
			    PenColor=Color.red;
			else if (s.getPen().equals("blue"))
	      PenColor=Color.blue;
			else if (s.getPen().equals("green"))
			    PenColor=Color.green;
			else PenColor=Color.white;
			
			g.setColor(PenColor);
			int oldx=s.getX();
			int oldy=s.getY();
			while (strokeElementsEnum.hasMoreElements())
			    {
				StrokeElement el=(StrokeElement) strokeElementsEnum.nextElement();
				if (el.getType()==0)
				    {
					if (PenColor!=Color.white)
					    {
						g.drawLine((int)(oldx*640.0/(right-left)), (int)(oldy*480.0/(bottom-top)), (int)((oldx+el.getX())*640.0/(right-left)), (int)((oldy+el.getY())*480.0/(bottom-top)));
						oldx+=el.getX();
						oldy+=el.getY();
					    }
					else if (s.getPen().equals("leraser"))
					    {
						g.fillOval((int)((oldx+el.getX())*640.0/(right-left))-20, (int)((oldy+el.getY())*480.0/(bottom-top))-20, 40, 40);
						oldx+=el.getX();
						oldy+=el.getY();
					    }
					else
					    {
						g.fillOval((int)((oldx+el.getX())*640.0/(right-left))-5, (int)((oldy+el.getY())*480.0/(bottom-top))-5, 10, 10);
						oldx+=el.getX();
						oldy+=el.getY();
					    }
				    }
				else
				    {
					if (PenColor!=Color.white)
					    {
						g.drawLine((int)(oldx*640.0/(right-left)), (int)(oldy*480.0/(bottom-top)), (int)(el.getX()*640.0/(right-left)), (int)(el.getY()*480.0/(bottom-top)));
						oldx=el.getX();
						oldy=el.getY();
					    }
					else if (s.getPen().equals("leraser"))
					    {
						g.fillOval((int)(el.getX()*640.0/(right-left))-20, (int)(el.getY()*480.0/(bottom-top))-20, 40, 40);
						oldx=el.getX();
						oldy=el.getY();
					    }
					else
					    {
						g.fillOval((int)(el.getX()*640.0/(right-left))-5, (int)(el.getY()*480.0/(bottom-top))-5, 10, 10);
						oldx=el.getX();
						oldy=el.getY();
					    }
					
				    }
			    }
		    }
	    }
	if (current!=null)
	    {
		Color PenColor;
		Enumeration strokeElementsEnum=current.elements();
		if (current.getPen().equals("black"))
		    PenColor=Color.black;
		else if (current.getPen().equals("red"))
		    PenColor=Color.red;
		else if (current.getPen().equals("blue"))
		    PenColor=Color.blue;
		else if (current.getPen().equals("green"))
		    PenColor=Color.green;
		else PenColor=Color.white;
		
		g.setColor(PenColor);
		int oldx=current.getX();
		int oldy=current.getY();
		while (strokeElementsEnum.hasMoreElements())
		    {
			StrokeElement el=(StrokeElement) strokeElementsEnum.nextElement();
			if (el.getType()==0)
			    {
				if (PenColor!=Color.white)
				    {
					g.drawLine((int)(oldx*640.0/(right-left)), (int)(oldy*480.0/(bottom-top)), (int)((oldx+el.getX())*640.0/(right-left)), (int)((oldy+el.getY())*480.0/(bottom-top)));
					oldx+=el.getX();
					oldy+=el.getY();
				    }
				else if (current.getPen().equals("leraser"))
				    {
					g.fillOval((int)((oldx+el.getX())*640.0/(right-left))-20, (int)((oldy+el.getY())*480.0/(bottom-top))-20, 40, 40);
					oldx+=el.getX();
					oldy+=el.getY();
				    }
				else 
				    {
					g.fillOval((int)((oldx+el.getX())*640.0/(right-left))-5, (int)((oldy+el.getY())*480.0/(bottom-top))-5, 10, 10);
					oldx+=el.getX();
					oldy+=el.getY();
				    }
				
			    }
			else
			    {
				if (PenColor!=Color.white)
				    {
					g.drawLine((int)(oldx*640.0/(right-left)), (int)(oldy*480.0/(bottom-top)), (int)(el.getX()*640.0/(right-left)), (int)(el.getY()*480.0/(bottom-top)));
					oldx=el.getX();
					oldy=el.getY();
				    }
				else if (current.getPen().equals("leraser"))
				    {
					g.fillOval((int)(el.getX()*640.0/(right-left))-20, (int)(el.getY()*480.0/(bottom-top))-20, 40, 40);
					oldx=el.getX();
					oldy=el.getY();
				    }
				else 
				    {
					g.fillOval((int)(el.getX()*640.0/(right-left))-5, (int)(el.getY()*480.0/(bottom-top))-5, 10, 10);
					oldx=el.getX();
					oldy=el.getY();
				    }
			    }
		    }
	    }
    }
    
    public void printExtents()
    {
	if (owner.debug)
	    System.out.println("Left: "+left+"; Right: "+right+"; Top: "+top+"; Bottom: "+bottom+"; Width: "+width);
    }
}

class SBDSPGauge extends Dialog
{
    private int value=0;
    private Label valueLbl=new Label("                                                       ");
    public SBDSPGauge(Frame owner)
    {
	super(owner, "Downloading DSP", false);
	Toolkit t=Toolkit.getDefaultToolkit();
	setLocation((t.getScreenSize().width-312)/2, (t.getScreenSize().height-50)/2);
	setSize(312, 50);
	add(valueLbl);
    }
    
    public void increment()
    {
	value++;
	String temp=new String("[");
	int i=0;
	for (; i< value; i++)
	    temp=temp.concat("*");
	for (; i< 54; i++)
	    temp=temp.concat(" ");
	temp=temp.concat("]");
	valueLbl.setText(temp);
    }
}

class Stroke
{
    private int time;
    private int x;
    private int y;
    private String pen;
    private Vector vector;
    
    public Stroke (int time, int x, int y, String pen)
    {
	this.time=time;
	this.x=x;
	this.y=y;
	this.pen=pen;
	vector=new Vector(100, 100);
    }
    
    public void add (StrokeElement e)
    {
	vector.addElement((Object) e);
    }
    
    public Enumeration elements()
    {
	return vector.elements();
    }
    
    public int getTime()
    {
	return time;
    }
    
    public int getX()
    {
	return x;
    }
    
    public int getY()
    {
	return y;
    }
    
    public String getPen()
    {
	return pen;
    }
}

class StrokeElement
{
    private int x;
    private int y;
    private int type;
    // 0 - DeltaDraw
    // 1 - Draw
    
    public StrokeElement(int x, int y, int type)
    {
	this.x=x;
	this.y=y;
	this.type=type;
    }
    
    public int getX()
    {
	return x;
    }
    
    public int getY()
    {
	return y;
    }
    
    public int getType()
    {
	return type;
    }
}
  
  

  
  
