/*******************************************************************
 *
 *    FILE:			progress.c			
 *
 *    DESCRIPTION:	Contains the SBDownShowProgress() function, which 
 *					is the place to make application specific changes
 *					regarding how the rest of the app is informed of 
 *					download progress.
 *
 *    HISTORY:    	
 *		Dec. 7, 1994:	Created - Guy Townsend/Microfield Graphics
 *
 *******************************************************************/

/** include files **/
#include <assert.h>
#include <stdio.h>
#include "sbhw.h"
#include "progress.h"
#include "sbout.h"

/** local definitions **/

/** external functions **/

/** external data **/

/** internal functions **/


/*---------------------------------------------------------------------------
 ** SBDownShowProgress()
 
 *  PARAMETERS:		
		Command: 		What kind of update to do to the display.

		PercentDone:	Percent of the download done, in whole numbers.

 *  DESCRIPTION:	
		Provides a single place from which to transfer information about 
		the progress of the download to the main program. Since a failure
		will return an error string from SBDownload(), error messages
		are not needed. This function provides a way to return messages
		about the success of the download at various stages, especially
		how far along the progress is.
		
		PERCENT_RESOLUTION is defined to whatever level of resolution you
		want sent to the main program. 
		 
 *  RETURNS:
		Nothing. 		
--------------------------------------------------------------------------*/
void SBDownShowProgress(int Command, int PercentDone)
{
	char buf[80];
    static int last_percent = 0;

	assert( (PercentDone >=0) && (PercentDone <= 100) );
	
	/* Since last_percent is a static variable, we have to know
	   when to reset it, especially in the case of downloading
	   more than once per run of the program. */
	if( PercentDone < PERCENT_RESOLUTION )
		last_percent = 0;

	/* The SB_PutString function will normally be replaced with 
		your own routine 	*/
	switch (Command)
	{
		case SHOW_PERCENT:
		{
			/* control how often the display is updated */
			if(	(PercentDone - last_percent >= PERCENT_RESOLUTION) || 
				(PercentDone == 100) )
			{
				sprintf(buf,"%d %% Complete", PercentDone);
				SB_PutString(buf);
				last_percent = PercentDone;
			}
			break;
		}
		case SHOW_DOWNLOAD1_SUCCESS:
		{
			SB_PutString("Download of first file okay");
			break;
		}
		case SHOW_DOWNLOAD2_SUCCESS:
		{
			SB_PutString("Download of second file okay");
			break;
		}
		case SHOW_DOWNLOAD3_SUCCESS:
		{
			SB_PutString("Download of third file okay");
			break;
		}
	
	}	
	
}

