/*******************************************************************
 *
 *    FILE:			sbtest.c
 *
 *    DESCRIPTION:	
 *   	Test application for softboard functions under UNIX/NEXT. 
 *
 *    HISTORY:    	
 *		Sept. 8, 1994:	First Release - Guy Townsend / Microfield
 * 
 *		Sept. 22, 1994:	Cleaned up a bit, and just made a simple 
 * 	 					example with an infinite loop for now to
 *						check communication and protocol handling.
 *
 *******************************************************************/

#include <stdio.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <stdlib.h>
#include <unistd.h>
#include <fcntl.h>
#include <netinet/in.h>

#include "sbhw.h"
#include "sbdown.h"
#include "sbdata.h"
#include "sbtags.h"	

/* output routines header */
#include "sbout.h"		
int insocket, inclient, cli_len, outsocket;
int counter=0;
struct sockaddr_in in_addr, cli_addr;
struct sockaddr_in local_addr, remote_addr;

char inmesg[256];
char outmesg[255];

void main(int argv, char* argc[])
{
	BOOL done;
	BOOL checkport=FALSE;
	int mask;
        int port; /* Local port */
	char remoteip[255];
	int remoteport;        
	/* SBShowDebugOutput = TRUE; */
	
	/* check the command line for the port to use */

	if (argv ==2)
	{
	  char temp[255];	
	  char g;
	  FILE *conf=fopen("sbdriver.conf","r");
	  if (conf==NULL)
	    {
	      fprintf(stderr, "Error reading configuration file\n");
	      exit(1);
	    }
	  fscanf(conf, "%[^0-9]s", temp);
	  fscanf(conf, "%d", &port);
	  fscanf(conf, "%c", &g);
          fscanf(conf, "%[^0-9]s", temp);
          fscanf(conf, "%[^\n]s", remoteip);
          fscanf(conf, "%c", &g);
          fscanf(conf, "%[^0-9]s", temp);
          fscanf(conf, "%d", &remoteport);
	  printf("Using local port %d, remote ip %s, remote port %d\n", port, remoteip, remoteport);
	  fclose(conf);
	  if ((insocket=socket(AF_INET, SOCK_STREAM, 0)) < 0)
	    {
	      fprintf(stderr, "Failed to create the in socket\n");
	      exit(1);
	    }
	  bzero((char *) & in_addr, sizeof(in_addr));
	  in_addr.sin_family=AF_INET;
	  in_addr.sin_addr.s_addr=htonl(INADDR_ANY);
	  in_addr.sin_port=htons(port);
	  if (bind(insocket, (struct sockaddr *) &in_addr, sizeof(in_addr)) < 0)
	    {
	      fprintf(stderr, "Failed to bind the in socket\n");
	      exit(1);
	    }
	  
	  listen(insocket, 1);
	  cli_len=sizeof(cli_addr);
	  if ((inclient=accept(insocket, (struct sockaddr *) &cli_addr, &cli_len))<0)
	    {
	      fprintf(stderr, "Failed to accept the in socket\n");
	      exit(1);
	    }

	  mask=fcntl(inclient, F_GETFL);
	  mask=mask | O_NONBLOCK;
	  fcntl(inclient, F_SETFL, mask);

	  while (1)
	    {
	      int len;
	      if ((len=recv(inclient, inmesg, sizeof(inmesg), 0))>0)
		{
		  inmesg[len]='\0';
		  printf("==> [%s]\n", inmesg);
		  if (!strcmp(inmesg, "READY"))
		    {
		      bzero((char *)&remote_addr, sizeof(remote_addr));
		      remote_addr.sin_family=AF_INET;
                      remote_addr.sin_addr.s_addr=inet_addr(remoteip); 
		      remote_addr.sin_port=htons(remoteport);

		      if ((outsocket=socket(AF_INET, SOCK_STREAM, 0))<0)
			{
			  fprintf(stderr, "Can't open outgoing socket!\n");
			  exit(1);
			}
		      if (connect(outsocket, (struct sockaddr *)&remote_addr, sizeof(remote_addr))<0)
			{
			  fprintf(stderr, "Can't connect!\n");
			  exit(1);
			}

		      strcpy(outmesg, "READY");
		      send(outsocket, outmesg, strlen(outmesg), 0);
		    }
		  if (!strcmp(inmesg, "QUERY_SOFTBOARD_STATUS"))
		    {
		      if (CheckDownloadStatus(argc[1]))
			{
			  strcpy(outmesg, "QUERY_SOFTBOARD_STATUS_ACTIVE");
			  send(outsocket, outmesg, strlen(outmesg), 0);
			}
		      else
			{
			  strcpy(outmesg, "QUERY_SOFTBOARD_STATUS_NOTACTIVE");
			  send(outsocket, outmesg, strlen(outmesg), 0);
			}
		    }
		  if (!strcmp(inmesg, "DOWNLOAD_DSP"))
		    {
		      /* start a download on the passed port */
		      done = DoDownload( argc[1] );
		      
		      /* only talk to the board if the download worked */
		      if (!done)
			{
			  fprintf(stderr, "Error Starting up SoftBoard\n");
			  exit(1);
			}
		      SBWait(5000); /* wait for board startup */
		      strcpy(outmesg, "DOWNLOAD_DSP_DONE");
		      send(outsocket, outmesg, strlen(outmesg), 0);
		    }
		  if (!strcmp(inmesg, "START_RECORDING"))
		    {
		      /* Start recording pen data and sending it back to us. */
		      SBDataSendCommand( TAG_START_RECORDING, 0 );
		      checkport=TRUE;
		    }
		  if (!strcmp(inmesg, "STOP_RECORDING"))
		    {
		      SBDataSendCommand( TAG_STOP_RECORDING, 0 );
		      checkport=FALSE;
		    }
		  if (!strcmp(inmesg, "SOFTBOARD_EXTENTS"))
		    {
		      SBDataSendCommand(TAG_SOFTBOARD_EXTENTS, 0);
		    }
		  if (!strcmp(inmesg, "BYE"))
		    {
		      SBDataSendCommand( TAG_STOP_RECORDING, 0 );
		      exit(0);
		    }
		}
	      if (checkport)
		{
		  SBDataCheckPort();
		  SBWait(100);
		}
	    }
	}	                        
	else
	  {
	    printf("\nUsage: sbdriver /dev/tty_device_name \n\n"); 
	    exit(0);
	  }
	
		


	/* These really need to be called when before the program exits, */
	/* but without a user interface, that isn't a way to do it yet. */

	TestClose();

}

				
/***** SB_ClearOutputWindow()  ******/
/*	Clears the output window */
void SB_ClearOutputWindow()
{
	/* clear the screen ? */	
} 


/***** SB_ShowError() ******/ 
/* 	Shows an error message in a dialog box */
void SB_ShowError( char* ErrorString )
{
	fprintf(stderr, "Error: %s \n", ErrorString);
}                            


/***** SB_PutString() ******/ 
/* puts a string to the output window */
void SB_PutString( char* String )
{
	fprintf(stdout, "%s\n", String);
}



