/*******************************************************************
 *
 *    FILE:			sbhw.c
 *
 *    DESCRIPTION:	
 *			Contains hardware and/or operating system dependent code
 *			that is used by other routines for interacting with a 
 *			SoftBoard.
 *
 *
 *    HISTORY:
 *    	Sept 8, 1994:	First UNIX release - Guy Townsend 
 *
 *    	Sept 22, 1994:	Updated to include NEXT support. - GAT
 *
 *******************************************************************/

/** include files **/

#include <stdio.h>
#include "sbhw.h"

/*---------------------------------------------------------------------------
 ** SBTrace()
 
 *  PARAMETERS:		
		Message:	Pointer to a string to be printed to the debug output.

 *  DESCRIPTION:	
		Prints a debug message to a debug output, IF gShowDebugOutput is
		TRUE. The implementation is OS dependent.
 
 *  RETURNS:	Nothing.
 		
--------------------------------------------------------------------------*/
void SBTrace( char *Message )
{

	if(!SBShowDebugOutput)
		return;
	else
	{
		fprintf(stdout,"%s \n", Message); 
	}
}


/*---------------------------------------------------------------------------
 ** SBReleaseControl()
 
 *  PARAMETERS:		None

 *  DESCRIPTION:	
		This function is called to release control back to the operating
		system temporarily. Especially needed for environments like
		MS Windows, which is cooperatively multitasking. Highly system
		dependent.

		NOTE: Not used for standard command-line UNIX, so this is almost 
		an empty function call. Something like this will be needed for
		an most GUI environments.
 
 *  RETURNS:	Value of the SBAbortFlag, which is an application wide
 				global variable. This flag may get set to TRUE in this
 				routine or during message handling whie this routine is
 				operating. If so, the calling function should abort.
 				
--------------------------------------------------------------------------*/
BOOL SBReleaseControl(void)
{
	return( SBAbortFlag);
}
 

/*---------------------------------------------------------------------------
 ** SBWait()
 
 *  PARAMETERS:		
		Milliseconds: how long to wait

 *  DESCRIPTION:	
		Waits for the specified number of milliseconds. Very platform
		specific. Most UNIX variants seem to have some method of timing
		milliseconds range events. Two are given here. Other platforms 
		may have to use the ftimefunction and the millitm member of the 
		timeb struct to do the millisecond level timing.
		 
		Slightly longer wait times will not usually be a problem, but
		shorter ones will. However, significantly longer periods will 
		slow the program, especially the download.
  
 *  RETURNS:	Nothing
 		
--------------------------------------------------------------------------*/
void SBWait( unsigned long Milliseconds)
{	
	usleep( (long) Milliseconds * 1000); /* param is microseconds */
}


